<?php

namespace Drupal\Tests\acquia_contenthub_subscriber\Kernel\EventSubscriber\EntityDataTamper;

use Acquia\ContentHubClient\CDF\CDFObject;
use Acquia\ContentHubClient\CDFDocument;
use Drupal\acquia_contenthub\Event\EntityDataTamperEvent;
use Drupal\acquia_contenthub_subscriber\EventSubscriber\EntityDataTamper\DisabledEntity;
use Drupal\acquia_contenthub_subscriber\Libs\InterestList\InterestListTracker;
use Drupal\acquia_contenthub_subscriber\SubscriberTracker;
use Drupal\Core\Entity\EntityInterface;
use Drupal\depcalc\DependencyStack;
use Drupal\depcalc\DependentEntityWrapper;
use Drupal\KernelTests\KernelTestBase;
use Prophecy\Argument;

/**
 * Tests DisabledEntity event subscriber.
 *
 * @coversDefaultClass \Drupal\acquia_contenthub_subscriber\EventSubscriber\EntityDataTamper\DisabledEntity
 *
 * @package acquia_contenthub_subscriber
 */
class DisabledEntityTest extends KernelTestBase {

  /**
   * Webhook uuid.
   */
  protected const WEBHOOK_UUID = 'webhook-uuid';

  /**
   * Entity uuid.
   */
  protected const ENTITY_UUID = '00000000-0000-0000-0000-000000000001';

  /**
   * Entity uuid 2.
   */
  protected const ENTITY_UUID_2 = '00000000-0000-0000-0000-000000000002';

  /**
   * Subscriber tracker mock.
   *
   * @var \Drupal\acquia_contenthub_subscriber\SubscriberTracker|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $subscriberTracker;

  /**
   * SUT.
   *
   * @var \Drupal\acquia_contenthub_subscriber\EventSubscriber\EntityDataTamper\DisabledEntity
   */
  protected DisabledEntity $sut;

  /**
   * Interest list tracker mock.
   *
   * @var \Drupal\acquia_contenthub_subscriber\Libs\InterestList\InterestListTracker|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $interestListTracker;

  /**
   * Disabled entities.
   *
   * @var array
   */
  public static array $disabledEntities = [];

  /**
   * {@inheritDoc}
   *
   * @throws \Exception
   */
  public function setUp(): void {
    parent::setUp();
    $this->interestListTracker = $this->prophesize(InterestListTracker::class);
    $this->subscriberTracker = $this->prophesize(SubscriberTracker::class);
    $this->sut = new DisabledEntity($this->subscriberTracker->reveal(), $this->interestListTracker->reveal());
  }

  /**
   * @covers ::getDisabledEntities
   */
  public function testGetDisabledEntitiesFromService(): void {
    $disabled_entities_mock = [self::ENTITY_UUID => ['interest_data']];
    $this->interestListTracker
      ->getDisabledEntities()
      ->willReturn([self::ENTITY_UUID]);
    $this->subscriberTracker
      ->setStatusByUuids([self::ENTITY_UUID], SubscriberTracker::AUTO_UPDATE_DISABLED)
      ->shouldBeCalled();
    $disabled_entities = $this->sut->getDisabledEntities();
    $this->assertEquals(array_keys($disabled_entities_mock), $disabled_entities);
  }

  /**
   * @covers ::onDataTamper
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function testOnDataTamper(): void {
    $disabled_entities_mock = [self::ENTITY_UUID];
    $this->interestListTracker
      ->getDisabledEntities()
      ->shouldBeCalled()
      ->willReturn($disabled_entities_mock);
    $cdf_document = $this->prophesize(CDFDocument::class);
    $cdf_object1 = new CDFObject('', self::ENTITY_UUID, '', '', '', []);
    $cdf_object2 = new CDFObject('', self::ENTITY_UUID_2, '', '', '', []);
    $cdf_document
      ->getEntities()
      ->shouldBeCalled()
      ->willReturn([$cdf_object1, $cdf_object2]);
    $stack = $this->prophesize(DependencyStack::class);
    $stack->addDependency(Argument::type(DependentEntityWrapper::class), Argument::type('bool'))
      ->shouldBeCalledOnce()
      ->will(function ($args) {
        /** @var \Drupal\depcalc\DependentEntityWrapper $wrapper */
        $wrapper = $args[0];
        self::$disabledEntities[] = $wrapper->getUuid();
      });
    $event = new EntityDataTamperEvent($cdf_document->reveal(), $stack->reveal());
    $entity = $this->prophesize(EntityInterface::class);
    $entity
      ->uuid()
      ->shouldBeCalled()
      ->willReturn(self::ENTITY_UUID);
    $entity->getEntityTypeId()
      ->shouldBeCalled()
      ->willReturn('');
    $entity->id()
      ->shouldBeCalled()
      ->willReturn('');
    $entity->toArray()
      ->shouldBeCalled()
      ->willReturn([]);
    $this->subscriberTracker
      ->getStatusByUuid(self::ENTITY_UUID)
      ->willReturn(SubscriberTracker::AUTO_UPDATE_DISABLED);
    $this->subscriberTracker
      ->setStatusByUuids([self::ENTITY_UUID], SubscriberTracker::AUTO_UPDATE_DISABLED)
      ->shouldBeCalled();
    $this->subscriberTracker
      ->setStatusByUuids([self::ENTITY_UUID_2], SubscriberTracker::AUTO_UPDATE_DISABLED)
      ->shouldNotBeCalled();
    $this->subscriberTracker
      ->getEntityByRemoteIdAndHash(self::ENTITY_UUID)
      ->shouldBeCalled()
      ->willReturn($entity->reveal());
    $this->subscriberTracker
      ->getEntityByRemoteIdAndHash(self::ENTITY_UUID_2)
      ->shouldNotBeCalled();
    $this->sut->onDataTamper($event);
    $this->assertContains(self::ENTITY_UUID, self::$disabledEntities);
    $this->assertNotContains(self::ENTITY_UUID_2, self::$disabledEntities);
  }

}
