<?php

namespace Drupal\acquia_contenthub_translations;

use Drupal\acquia_contenthub_translations\Data\EntityTranslationsDAOInterface;
use Drupal\acquia_contenthub_translations\Data\TrackedEntity;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\Upsert;

/**
 * Provides an abstraction layer between the DAOs and upper layer.
 *
 * Additionally, the manager provides a simple caching mechanism as well.
 * The cache is stored in memory.
 */
class EntityTranslationsManager implements EntityTranslationManagerInterface {

  /**
   * The entity translations DAO.
   *
   * @var \Drupal\acquia_contenthub_translations\Data\EntityTranslationsDAOInterface
   */
  protected $translations;

  /**
   * The entity translations tracker DAO.
   *
   * @var \Drupal\acquia_contenthub_translations\Data\EntityTranslationsDAOInterface
   */
  protected $tracker;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Cache for already retrieved items.
   *
   * @var \Drupal\acquia_contenthub_translations\Data\TrackedEntity[]
   */
  private static $cache;

  /**
   * Constructs a new object.
   *
   * @param \Drupal\acquia_contenthub_translations\Data\EntityTranslationsDAOInterface $translations
   *   The entity translations DAO.
   * @param \Drupal\acquia_contenthub_translations\Data\EntityTranslationsDAOInterface $tracker
   *   The entity translations tracker DAO.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(EntityTranslationsDAOInterface $translations, EntityTranslationsDAOInterface $tracker, Connection $database) {
    $this->translations = $translations;
    $this->tracker = $tracker;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public function getTrackedEntity(string $entity_uuid): ?TrackedEntity {
    if (isset(self::$cache[$entity_uuid])) {
      return self::$cache[$entity_uuid];
    }

    $query = $this->database->select($this->tracker::tableName(), 'et')
      ->fields('et')
      ->fields('ett', ['langcode', 'operation_flag'])
      ->condition('et.entity_uuid', $entity_uuid);
    $query->leftJoin($this->translations::tableName(), 'ett', '[ett].[entity_uuid] = [et].[entity_uuid]');
    $rows = $query
      ->condition('et.entity_uuid', $entity_uuid)
      ->execute()
      ->fetchAll(\PDO::FETCH_ASSOC);

    if (empty($rows)) {
      return NULL;
    }

    $normalized = [];
    $first = current($rows);
    $normalized['uuid'] = $first['entity_uuid'];
    $normalized['type'] = $first['entity_type'];
    $normalized['original_default_language'] = $first['original_default_language'];
    $normalized['default_language'] = $first['default_language'];
    $normalized['created'] = $first['created'];
    $normalized['changed'] = $first['changed'];
    foreach ($rows as $row) {
      if (isset($row['langcode'])) {
        $normalized['languages'][$row['langcode']] = (int) $row['operation_flag'];
      }
    }

    $entity = new TrackedEntity($this, $normalized);
    self::$cache[$entity_uuid] = $entity;

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteTrackedEntity(string $entity_uuid): void {
    $this->translations->deleteByUuid($entity_uuid);
    $this->tracker->deleteByUuid($entity_uuid);
  }

  /**
   * {@inheritdoc}
   */
  public function trackEntity(string $uuid, string $type, string $original_default_langcode, string $default_langcode): TrackedEntity {
    $time = time();
    $this->tracker->insert(
      [
        'entity_uuid' => $uuid,
        'entity_type' => $type,
        'original_default_language' => $original_default_langcode,
        'default_language' => $default_langcode,
        'created' => $time,
        'changed' => $time,
      ]
    );
    $values = [
      'uuid' => $uuid,
      'type' => $type,
      'original_default_language' => $original_default_langcode,
      'default_language' => $default_langcode,
      'created' => $time,
      'changed' => $time,
    ];
    $entity = new TrackedEntity($this, $values);
    self::$cache[$uuid] = $entity;
    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function trackMultiple(array $values): void {
    // An arbitrary length, but sufficient.
    $chunks = array_chunk($values, 100, TRUE);
    foreach ($chunks as $data) {
      $this->insertOrUpdateMultiple($data);
    }
  }

  /**
   * Inserts or updates multiple entities.
   *
   * @param array $values
   *   The entity values.
   */
  protected function insertOrUpdateMultiple(array $values): void {
    $tracked_entities = $this->getDataFromTrackingTable($values);
    $rows = [];
    foreach ($values as $value) {
      $entity = $tracked_entities[$value['entity_uuid']] ?? NULL;
      if ($entity) {
        $rows[] = $this->prepareUpdateRow($entity, $value);
        unset(self::$cache[$value['entity_uuid']]);
        continue;
      }
      $rows[] = $this->prepareInsertRow($value);
    }
    $upsert_query = $this->prepareUpsertQuery($rows);
    $upsert_query->execute();
  }

  /**
   * Fetches data from tracking table.
   *
   * @param array $values
   *   Entity values.
   *
   * @return array
   *   Data from tracking table.
   */
  protected function getDataFromTrackingTable(array $values): array {
    $entity_uuids = array_column($values, 'entity_uuid');
    $select_query = $this->database->select($this->tracker::tableName(), 't')->fields('t', [
      'entity_uuid',
      'entity_type',
      'original_default_language',
      'default_language',
      'created',
      'changed',
    ])->condition('entity_uuid', $entity_uuids, 'IN');
    $tracked_entities = $select_query->execute()->fetchAllAssoc('entity_uuid');
    return json_decode(json_encode($tracked_entities), TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function trackTranslation(string $uuid, string $langcode, int $operation = self::NO_ACTION): void {
    $this->translations->insert([
      'entity_uuid' => $uuid,
      'langcode' => $langcode,
      'operation_flag' => $operation,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function removeTranslation(string $uuid, string $langcode): void {
    $this->translations->deleteBy('entity_uuid', $uuid,
      [['field_name' => 'langcode', 'value' => $langcode, 'operator' => '=']]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function updateTrackedEntity(TrackedEntity $entity): void {
    $changed = $entity->getChangedValues();
    if (empty($changed)) {
      return;
    }

    // Update translations table.
    if (isset($changed['languages'])) {
      $langs = $changed['languages'];
      $original_langs = $entity->languages();
      $removable = array_diff_key($original_langs, $langs);
      foreach (array_keys($removable) as $lang) {
        $this->removeTranslation($entity->uuid(), $lang);
      }

      foreach ($langs as $lang => $operation_flag) {
        if (!isset($original_langs[$lang])) {
          $this->trackTranslation($entity->uuid(), $lang, $operation_flag);
          continue;
        }
        if ($operation_flag === $original_langs[$lang]) {
          continue;
        }
        $this->translations->update($entity->uuid(), [
          'values' => ['operation_flag' => $operation_flag],
          'langcode' => $lang,
        ]);
      }
      // Tracker doesn't accept languages fields therefore needs to be unset.
      unset($changed['languages']);
    }

    $this->tracker->update($entity->uuid(), ['values' => $changed]);

    unset(self::$cache[$entity->uuid()]);
  }

  /**
   * Prepares upsert query for ::trackMultiple.
   *
   * @param array $values
   *   Values to upsert.
   *
   * @return \Drupal\Core\Database\Query\Upsert
   *   Upsert query with rows prepared.
   */
  protected function prepareUpsertQuery(array $values): Upsert {
    $upsert_query = $this->database->upsert($this->tracker::tableName());
    $upsert_query->key('entity_uuid');
    $upsert_query->fields([
      'entity_uuid',
      'default_language',
      'entity_type',
      'created',
      'changed',
      'original_default_language',
    ]);
    foreach ($values as $value) {
      $upsert_query->values($value);
    }

    return $upsert_query;
  }

  /**
   * Prepares update row for upsert query.
   *
   * @param array $original_data
   *   Original data.
   * @param array $value
   *   New values.
   *
   * @return array
   *   Row for upsert query.
   */
  protected function prepareUpdateRow(array $original_data, array $value): array {
    // Only allow default_language and changed value to update.
    // Because other values cannot be updated once entity created.
    $row['entity_uuid'] = $original_data['entity_uuid'];
    $row['entity_type'] = $original_data['entity_type'];
    $row['created'] = $original_data['created'];
    $row['changed'] = $value['changed'] ?? time();
    $row['default_language'] = $value['default_language'] ?? $original_data['default_language'];
    $row['original_default_language'] = $original_data['original_default_language'];

    return $row;
  }

  /**
   * Prepares insert row for upsert query.
   *
   * @param array $data
   *   Data to insert.
   *
   * @return array
   *   Insert row for upsert query.
   */
  protected function prepareInsertRow(array $data): array {
    $time = time();
    return [
      'entity_uuid' => $data['entity_uuid'],
      'entity_type' => $data['entity_type'],
      'original_default_language' => $data['original_default_language'],
      'default_language' => $data['default_language'],
      'created' => $time,
      'changed' => $time,
    ];
  }

}
