<?php

namespace Drupal\acquia_contenthub_translations\EventSubscriber\EntityDataTamper;

use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Event\EntityDataTamperEvent;
use Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry;
use Drupal\acquia_contenthub_translations\EntityTranslationManagerInterface;
use Drupal\Core\Language\LanguageInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Normalizes field values of CDF if the original language has changed.
 */
class NormalizeFieldValues implements EventSubscriberInterface {

  /**
   * The translation manager service.
   *
   * @var \Drupal\acquia_contenthub_translations\EntityTranslationManagerInterface
   */
  protected EntityTranslationManagerInterface $translationManager;

  /**
   * Service to access handler information regarding entities.
   *
   * @var \Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry
   */
  protected HandlerRegistry $handlerRegistry;

  /**
   * Constructs a new object.
   *
   * @param \Drupal\acquia_contenthub_translations\EntityTranslationManagerInterface $manager
   *   The translation manager service.
   * @param \Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry $handler_registry
   *   Handler registry service.
   */
  public function __construct(EntityTranslationManagerInterface $manager, HandlerRegistry $handler_registry) {
    $this->translationManager = $manager;
    $this->handlerRegistry = $handler_registry;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaContentHubEvents::ENTITY_DATA_TAMPER][] = 'onDataTamper';
    return $events;
  }

  /**
   * Tamper with CDF data before its imported.
   *
   * @param \Drupal\acquia_contenthub\Event\EntityDataTamperEvent $event
   *   The data tamper event.
   */
  public function onDataTamper(EntityDataTamperEvent $event): void {
    foreach ($event->getCdf()->getEntities() as $uuid => $object) {
      $metadata = $object->getMetadata();

      $tracked = $this->translationManager->getTrackedEntity($uuid);
      if (!$tracked) {
        continue;
      }

      if (!isset($metadata['translatable'])) {
        continue;
      }

      $type = $object->getAttribute('bundle')->getValue()[LanguageInterface::LANGCODE_NOT_SPECIFIED];
      if (!$this->shouldProcess($tracked->type(), $metadata['translatable'], $type)) {
        continue;
      }

      $lang = $tracked->defaultLanguage();
      $orig = $tracked->originalDefaultLanguage();
      if ($lang === $orig) {
        continue;
      }

      $field_values = json_decode(base64_decode($metadata['data']), TRUE);
      $normalized_values = [];
      foreach ($field_values as $name => $value) {
        // We give preference for default language ($lang) instead of
        // original default language ($orig). Because in case of
        // circular dependency normalizer runs twice. During first run
        // field values are replaced with original default language.
        $field_value = $value['value'][$lang] ?? $value['value'][$orig];
        $normalized_values[$name]['value'][$lang] = $field_value;
      }
      $metadata['data'] = base64_encode(json_encode($normalized_values));
      $object->setMetadata($metadata);
    }
  }

  /**
   * Checks whether the provided entity type should be processed by normalizer.
   *
   * @param string $entity_type
   *   The entity type to check.
   * @param bool $translatable
   *   Flag whether the entity type is translatable or not by default.
   * @param string|null $bundle
   *   The bundle to check.
   *
   * @return bool
   *   TRUE if the entity should be processed.
   */
  public function shouldProcess(string $entity_type, bool $translatable, ?string $bundle = NULL): bool {
    if (!$translatable) {
      return TRUE;
    }
    return $this->handlerRegistry->isInOverridden($entity_type, $bundle) && $this->handlerRegistry->has('flexible', $entity_type, $bundle);
  }

}
