<?php

namespace Drupal\Tests\acquia_contenthub_translations\Unit\EntityDataTamper\EventSubscriber;

use Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry;
use Drupal\acquia_contenthub_translations\EntityTranslationManagerInterface;
use Drupal\acquia_contenthub_translations\EventSubscriber\EntityDataTamper\NormalizeFieldValues;
use Drupal\Core\Config\Config;
use Drupal\Tests\acquia_contenthub_translations\Traits\CdfGeneratorForTranslationsTrait;
use Drupal\Tests\UnitTestCase;

/**
 * Tests whether syndication is active while parsing the CDF.
 *
 * @coversDefaultClass \Drupal\acquia_contenthub_translations\EventSubscriber\EntityDataTamper\NormalizeFieldValues
 *
 * @requires module depcalc
 *
 * @group acquia_contenthub_translations
 *
 * @package Drupal\Tests\acquia_contenthub_translations\Unit
 */
class NormalizeFieldValuesTest extends UnitTestCase {

  use CdfGeneratorForTranslationsTrait;

  /**
   * Service to access handler information regarding entities.
   *
   * @var \Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry
   */
  protected HandlerRegistry $handlerRegistry;

  /**
   * The event subscriber under test.
   *
   * @var \Drupal\acquia_contenthub_translations\EventSubscriber\EntityDataTamper\NormalizeFieldValues
   */
  protected NormalizeFieldValues $normalizeFieldValues;

  /**
   * The acquia_contenthub_translations config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'acquia_contenthub',
    'acquia_contenthub_subscriber',
    'acquia_contenthub_translations',
    'depcalc',
    'language',
    'system',
    'user',
  ];

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $entityTranslationManager = $this->getMockBuilder(EntityTranslationManagerInterface::class)
      ->onlyMethods([])->getMock();
    $config = $this->prophesize(Config::class);
    $this->handlerRegistry = $this->getMockBuilder(HandlerRegistry::class)
      ->setConstructorArgs([
        $config->reveal(),
      ])
      ->getMock();
    $this->normalizeFieldValues = new NormalizeFieldValues($entityTranslationManager, $this->handlerRegistry);
  }

  /**
   * Tests shouldProcess() if entity is non-translatable.
   *
   * @covers \Drupal\acquia_contenthub_translations\EventSubscriber\EntityDataTamper\NormalizeFieldValues::shouldProcess()
   */
  public function testShouldProcessForNonTranslatable(): void {
    $sut = $this->normalizeFieldValues->shouldProcess('path_alias', FALSE);
    $this->assertTrue($sut);
  }

  /**
   * Tests shouldProcess().
   *
   * In this case, entity is translatable and Flexible handler is set,
   * but entity is not present in override_registry.
   */
  public function testShouldProcessIfOnlyFlexibleHandlerIsSet(): void {
    $this->handlerRegistry->method('isInOverridden')
      ->willReturn(FALSE);
    $this->handlerRegistry->method('has')
      ->willReturn(TRUE);
    $sut = $this->normalizeFieldValues->shouldProcess('path_alias', TRUE, 'path_alias');
    $this->assertFalse($sut);
  }

  /**
   * Tests shouldProcess().
   *
   * In this case, entity is translatable and is present in override_registry,
   * but Flexible handler is not set.
   */
  public function testShouldProcessIfOnlyOverRiddenRegistryIsSet(): void {
    $this->handlerRegistry->method('isInOverridden')
      ->willReturn(TRUE);
    $this->handlerRegistry->method('has')
      ->willReturn(FALSE);
    $sut = $this->normalizeFieldValues->shouldProcess('path_alias', TRUE, 'path_alias');
    $this->assertFalse($sut);
  }

  /**
   * Tests shouldProcess().
   *
   * In this case, entity is translatable and is present in override_registry,
   * and it's Flexible handler is also set.
   */
  public function testShouldProcessIfFlexibleHandlerAndOverRiddenRegistryIsSet(): void {
    $this->handlerRegistry->method('isInOverridden')
      ->willReturn(TRUE);
    $this->handlerRegistry->method('has')
      ->willReturn(TRUE);
    $sut = $this->normalizeFieldValues->shouldProcess('path_alias', TRUE, 'path_alias');
    $this->assertTrue($sut);
  }

}
