<?php

namespace Drupal\Tests\acquia_contenthub_translations\Unit;

use Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry;
use Drupal\Core\Config\Config;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * @coversDefaultClass \Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry
 *
 * @group acquia_contenthub_translations
 */
class HandlerRegistryTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The mocked acquia_contenthub_translations config.
   *
   * @var \Drupal\Core\Config\Config|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->config = $this->prophesize(Config::class);
  }

  /**
   * @covers ::has
   *
   * @dataProvider testEntityHasHandlersDataProvider
   */
  public function testEntityHasHandlers(array $data, string $handler_id, string $entity_type, string $bundle, bool $expected, string $message): void {
    $this->config->get(Argument::any([]))->willReturn($data);
    $registry = $this->newHandlerRegistry();
    $actual = $registry->has($handler_id, $entity_type, $bundle);
    $this->assertEquals($expected, $actual, $message);
  }

  /**
   * @covers ::isInOverridden
   *
   * @dataProvider testIsInOverriddenRegistryDataProvider
   */
  public function testIsInOverriddenRegistry(array $data, string $entity_type, string $bundle, bool $expected, string $message): void {
    $this->config->get(Argument::any([]))->willReturn($data);
    $registry = $this->newHandlerRegistry();
    $actual = $registry->isInOverridden($entity_type, $bundle);
    $this->assertEquals($expected, $actual, $message);
  }

  /**
   * Provides data for testIsInOverriddenRegistry.
   *
   * @return array
   *   The test cases.
   */
  public function testIsInOverriddenRegistryDataProvider(): array {
    return [
      [
        [
          'node:node',
        ],
        'node',
        '',
        TRUE,
        'Node entity type is overridden',
      ],
      [
        [
          'node:article',
        ],
        'node',
        'page',
        FALSE,
        'Page bundle of node is not overridden',
      ],
      [
        [
          'node:article',
          'paragraph:paragraph',
        ],
        'paragraph',
        'card',
        TRUE,
        'All type of paragraph are overridden',
      ],
    ];
  }

  /**
   * Provides data for testEntityHasHandlers.
   *
   * @return array
   *   The test cases.
   */
  public function testEntityHasHandlersDataProvider(): array {
    return [
      [
        [
          'node:node' => 'flexible',
        ],
        'flexible',
        'node',
        '',
        TRUE,
        'Node has flexible handler.',
      ],
      [
        [],
        'flexible',
        'node',
        '',
        FALSE,
        'Node is not recorded in the handler registry.',
      ],
      [
        [
          'taxonomy_term:tags' => 'flexible',
          'node:node' => 'removable',
        ],
        'flexible',
        'taxonomy_term',
        'tags',
        TRUE,
        'Taxonomy term has the flexible handler',
      ],
      [
        [
          'node:article' => 'flexible',
        ],
        'flexible',
        'node',
        'page',
        FALSE,
        'Node of type page does not have flexible handler',
      ],
    ];
  }

  /**
   * Instantiates a new HandlerRegistry.
   *
   * @return \Drupal\acquia_contenthub_translations\EntityHandler\HandlerRegistry
   *   The instance.
   */
  protected function newHandlerRegistry(): HandlerRegistry {
    return new HandlerRegistry($this->config->reveal());
  }

}
